package cn.kdan.cloud.pdf.office.account.service.impl;

import cn.kdan.cloud.pdf.office.account.mapper.LoginDeviceMapper;
import cn.kdan.cloud.pdf.office.account.model.LoginDevice;
import cn.kdan.cloud.pdf.office.account.model.LoginDeviceExample;
import cn.kdan.cloud.pdf.office.account.service.LoginDeviceService;
import cn.kdan.cloud.pdf.office.common.enums.ValidStatusEnum;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author tangxiangan
 */
@Service
public class LoginDeviceServiceImpl implements LoginDeviceService {

    @Resource
    private LoginDeviceMapper loginDeviceMapper;

    @Override
    public void create(LoginDevice loginDevice) {
        loginDevice.setValidFlag(ValidStatusEnum.VALID.value());
        loginDeviceMapper.insert(loginDevice);
    }


    @Override
    public List<LoginDevice> getByUserId(String userId, String appId) {
        LoginDeviceExample example = new LoginDeviceExample();
        LoginDeviceExample.Criteria criteria =  example.createCriteria().andUserIdEqualTo(userId).andValidFlagNotEqualTo(ValidStatusEnum.INVALID.value());
        if(!StringUtils.isEmpty(appId)){
            criteria.andAppIdEqualTo(appId);
        }
        return loginDeviceMapper.selectByExample(example);
    }

    @Override
    public void delete(String deviceSign, String userId, String appId, Boolean isDelete) {
        LoginDeviceExample example = new LoginDeviceExample();
        example.createCriteria().andUniqueSnEqualTo(deviceSign).andUserIdEqualTo(userId).andAppIdEqualTo(appId);
        if(isDelete){
            loginDeviceMapper.deleteByExample(example);
        }else{
            LoginDevice device = new LoginDevice();
            device.setValidFlag(ValidStatusEnum.INVALID.value());
            loginDeviceMapper.updateByExampleSelective(device,example);
        }
    }

    @Override
    public LoginDevice getByDeviceSignAndAppId(String deviceSign, String userId, String appId) {
        LoginDeviceExample example = new LoginDeviceExample();
        LoginDeviceExample.Criteria criteria = example.createCriteria().andUniqueSnEqualTo(deviceSign)
                .andAppIdEqualTo(appId);
        example.setOrderByClause("created_at DESC");
        if(!StringUtils.isEmpty(userId)){
            criteria.andUserIdEqualTo(userId);
        }
        List<LoginDevice> list = loginDeviceMapper.selectByExample(example);
        if(CollectionUtils.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }

    @Override
    public LoginDevice getById(String deviceId) {
        return loginDeviceMapper.selectByPrimaryKey(deviceId);
    }

}
