package cn.kdan.cloud.pdf.office.api.account.feign;

import cn.kdan.cloud.pdf.office.api.account.dto.*;
import cn.kdan.cloud.pdf.office.api.account.feign.hystrix.UserHystrix;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.common.config.FeignConfig;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.UserRegisterDTO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import com.github.pagehelper.PageInfo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Date;
import java.util.List;


@Component
@FeignClient(value = CommonConstant.ACCOUNT_PROVIDER_NAME + "/user" , configuration = FeignConfig.class,fallback = UserHystrix.class)
public interface UserApi {


    @GetMapping(value = "getByAppAccount")
    ResultMap<UserVO> getByAppAccount(@RequestParam("account") String account, @RequestParam("platformType") String platformType);

    @GetMapping(value = "getById")
    ResultMap<UserVO> getById(@RequestParam("id") String id);

    /**
     * 用户信息包括用户的会员信息
     *
     * @param id id
     * @return UserInfoVO
     */
    @GetMapping("getInfoById")
    ResultMap<UserInfoVO> getInfoById(@RequestParam("id") String id);


    /**
     * 用户信息包括用户的会员信息（新会员系统）
     *
     * @param id id
     * @return UserInfoVO
     */
    @GetMapping("getMemberInfoById")
    ResultMap<UserInfoVO> getMemberInfoById(@RequestParam("id") String id);

    @PostMapping(value = "register")
    ResultMap<Boolean> register(@RequestBody UserRegisterDTO userRegisterDTO);

    /**
     * 更新
     *
     * @param userVO
     */
    @PostMapping("/updateUser")
    ResultMap<Boolean> updateUser(@RequestBody UserVO userVO);

    /**
     * 支付成功之后修改用户状态
     *
     * @param updateUserForOrderDTO dto
     */
    @PostMapping(value = "updateUserForOrder")
    ResultMap<Boolean> updateUserForOrder(@RequestBody UpdateUserForOrderDTO updateUserForOrderDTO);


    @GetMapping("/page")
    ResultMap<PageInfo<UserPageVO>> page(@RequestParam(value = "page",required = false, defaultValue = "1") Integer page,
                                            @RequestParam(value = "pageSize",required = false, defaultValue = "10") Integer pageSize,
                                            @RequestParam(value = "selectType",required = false) String selectType,
                                            @RequestParam(value = "appIds",required = false) List<String> appIds,
                                            @RequestParam(value = "roles",required = false)  List<String> roles,
                                            @RequestParam(value = "accountTypes",required = false)  List<String> accountTypes,
                                            @RequestParam(value = "accountSources",required = false)  List<String> accountSources,
                                            @RequestParam(value = "payTypes",required = false)  List<String> payTypes,
                                            @RequestParam(value = "queryTimeType",required = false)  String queryTimeType,
                                            @RequestParam(value = "startTime",required = false) Date startTime,
                                            @RequestParam(value = "endTime",required = false)  Date endTime,
                                            @RequestParam(value = "queryString",required = false)  List<String> queryString);
    /**
     * 运营后台查看账号详情
     *
     * @param userId 用户id
     * @return ResultMap
     */
    @GetMapping("/detail")
    ResultMap<UserDetailVO> detail(@RequestParam("userId") String userId);


    /**
     * 后台强制注销用户
     *
     * @param logOffUserDTO
     */
    @PostMapping("/logOff")
    ResultMap<Boolean> logOff(@RequestBody LogOffUserDTO logOffUserDTO);


    /**
     * 用户自己手动注销
     *
     * @param code
     * @param appId
     * @return
     */
    @PostMapping("/logOffForUser")
    ResultMap<Boolean> logOffForUser(@RequestParam("code") String code, @RequestParam("userId")  String userId, @RequestParam("appId")  String appId);

    /**
     * 根据email查询用户
     *
     * @param email email
     * @return UserVO
     */
    @GetMapping("/getByEmail")
    ResultMap<UserVO> getByEmail(@RequestParam("email") String email);

    @GetMapping("/getInviteNum")
    ResultMap<Integer> getInviteNum(@RequestParam("userId") String userId);
}
