package cn.kdan.cloud.pdf.office.api.payment.bo;

import cn.kdan.cloud.pdf.office.api.payment.paypal.*;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.datatype.jsr310.deser.LocalDateTimeDeserializer;
import com.fasterxml.jackson.datatype.jsr310.ser.LocalDateTimeSerializer;
import lombok.Data;
import lombok.ToString;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/17
 */
@Data
@ToString
public class CreateSubscriptionsResultPaypalBO implements Serializable {

    /**
     * 订阅的计费详细信息。如果订阅曾经或处于活动状态，则会填充这些字段。
     */
    private SubscriptionBillingInfo billing_info;

    /**
     * The date and time, in Internet date and time format.
     * Seconds are required while fractional seconds are optional.
     */
    @JsonDeserialize(using = LocalDateTimeDeserializer.class)
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime create_time;

    /**
     * 订阅的自定义 ID。可以是发票 ID。
     */
    private String custom_id;

    /**
     * 订阅PayPal生成的 ID。
     */
    private String id;

    /**
     * 与请求相关的 HATEOAS 链接的数组。
     */
    private List<LinkDescription> links;
    /**
     * 内联计划详细信息。
     */
    private Plan plan;
    /**
     * The ID of the plan.
     */
    private String plan_id;
    /**
     * 指示订阅是否已重写任何计划属性
     */
    private Boolean plan_overridden;
    /**
     * 订阅中产品的数量。
     */
    private String quantity;
    /**
     * 金融交易的货币和金额，例如到期余额或付款。
     */
    private Money shipping_amount;
    /**
     * start_time
     */
    @JsonDeserialize(using = LocalDateTimeDeserializer.class)
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime start_time;
    /**
     * 订阅的状态。
     * <p>
     * 可能的值为：
     * <p>
     * APPROVAL_PENDING. 订阅已创建，但尚未获得买方批准。
     * APPROVED. 买方已批准订阅。
     * ACTIVE. 订阅处于活动状态。
     * SUSPENDED. 订阅已暂停。
     * CANCELLED. 订阅已取消。
     * EXPIRED. 订阅已过期。
     */
    private String status;
    /**
     * 订阅状态的原因或说明。
     */
    private String status_change_note;
    /**
     * status_update_time
     */
    @JsonDeserialize(using = LocalDateTimeDeserializer.class)
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime status_update_time;
    /**
     * 订户响应信息
     */
    private Subscriber subscriber;
    /**
     * update_time
     */
    @JsonDeserialize(using = LocalDateTimeDeserializer.class)
    @JsonSerialize(using = LocalDateTimeSerializer.class)
    private LocalDateTime update_time;


}
