package cn.kdan.cloud.pdf.office.api.payment.paypal;

import lombok.Data;
import lombok.ToString;

import java.io.Serializable;

/**
 * @author ComPDFKit-WPH 2023/1/16
 *
 * 应用程序上下文，用于在订阅审批过程中使用 PayPal 自定义付款人体验。
 *
 */
@Data
@ToString
public class ApplicationContext implements Serializable {
    /**
     * 客户取消付款后重定向客户的 URL。
     */
    private String cancel_url;
    /**
     * 客户批准付款后重定向客户的 URL。
     */
    private String return_url;
    /**
     * 覆盖PayPal网站上PayPal帐号中的商家名称的标签。
     */
    private String brand_name;
    /**
     * PayPal付款体验显示的页面的 BCP 47 格式区域设置。PayPal支持五个字符的代码。
     * 例如，、、、、da-DK,he-IL,id-ID,ja-JP,no-NO,pt-BR,ru-RU,sv-SE,th-TH,zh-CN,zh-HK,zh-TW
     * Pattern: .^[a-z]{2}(?:-[A-Z][a-z]{3})?(?:-(?:[A-Z]{2}))?$
     */
    private String locale;

    /**
     * 客户和商家的付款首选项。目前仅支持PayPal付款方式
     */
    private PaymentMethod payment_method;

    /**
     * 从中派生送货地址的位置。
     */
    private String shipping_preference;
    /**
     * 将标签名称配置为订阅同意体验或用于订阅同意体验。
     */
    private String user_action;

}
