package cn.kdan.cloud.pdf.office.common.aop;


import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * 在接口调用时打印日志的aop
 */
@Aspect
@Component
@Order(3)
public class CommonLogPrinterAspect {

    private Logger logger = LoggerFactory.getLogger(getClass());


    //申明一个切点 里面是 execution表达式
    @Pointcut("@within(org.springframework.stereotype.Controller) || @within(org.springframework.web.bind.annotation.RestController)")
    private void controllerAspect(){}


    //请求method前打印内容
    @Before(value = "controllerAspect()")
    public void methodBefore(JoinPoint joinPoint){
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if(requestAttributes == null){
            return;
        }
        HttpServletRequest request = requestAttributes.getRequest();

        // 打印请求内容
        String log =
                "\r\n===============请求内容===============\r\n"
                        + "请求地址" + request.getRequestURL().toString() + "\r\n"
                        + "请求方式:" + request.getMethod() + "\r\n"
                        + "请求类方法:" + joinPoint.getSignature() + "\r\n"
                        + "请求类方法参数:" + JsonUtils.getJsonString(joinPoint.getArgs()) + "\r\n"
            + "===============请求内容===============\r\n";
        logger.info(log);
    }


    //在方法执行完结后打印返回内容
    @AfterReturning(returning = "o",pointcut = "controllerAspect()")
    public void methodAfterReturing(Object o){
        String log =
                "\r\n--------------返回内容----------------\r\n"
                + "Response内容:" + JsonUtils.getJsonString(o) +"\r\n"
                + "--------------返回内容----------------\r\n";
        logger.info(log);
    }

}
