package cn.kdan.cloud.pdf.office.common.utils;


import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.PageVO;
import cn.kdan.cloud.pdf.office.common.pojo.params.QueryParams;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

public class QueryUtils {

    /**
     * 凭借like查询语句
     * @param string
     * @return
     */
    public static String generateLikeString(String string){
        try {
            //处理下%和_的问题，应该还有其他通配符
            string = string.replace(CommonConstant.STRING_PERCENT,"\\" + CommonConstant.STRING_PERCENT)
                    .replace(CommonConstant.STRING_UNDERLINE,"\\" + CommonConstant.STRING_UNDERLINE);
            string = URLDecoder.decode(URLEncoder.encode(string, CommonUtils.ENCODING_UTF8), CommonUtils.ENCODING_UTF8);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("unsupport encoding");
        }
        return "%"+string+"%";
    }

    /**
     * 处理分页查询参数 设置当前页、行数和排序字段规则
     * @param param
     * @param <T>
     */
    public static <T extends QueryParams> void handleParam(T param) {
        Integer pageNum = PageUtils.getParamToInt(param.getPage(), CommonConstant.PAGENUM);
        Integer pageSize = PageUtils.getParamToInt(param.getRows(), CommonConstant.PAGESIZE);
        String orderByClause = PageUtils.getOrderByClause(param.getSidx(), param.getSord(), CommonConstant.ORDERBYCLAUSE);
        param.setOrderByClause(orderByClause);
        param.setPage(pageNum);
        param.setRows(pageSize);
    }

    /**
     * 转换page的bean
     * @param pageInfo
     * @param converter
     * @param <T>
     * @param <V>
     * @return
     */
    public static <T,V> PageVO<T> convertPageInfoToVO(PageInfo<V> pageInfo, BeanConverter<V,T> converter){
        PageVO<T> pageVO = new PageVO<>();
        BeanUtils.copyProperties(pageInfo,pageVO);
        pageVO.setList(convertListToVO(pageInfo.getList(),converter));
        return pageVO;
    }


    /**
     * 转换list的bean
     * @param list
     * @param converter
     * @param <T>
     * @param <V>
     * @return
     */
    public static <T,V> List<T> convertListToVO(List<V> list,BeanConverter<V,T> converter){
        List<T> voList = new CopyOnWriteArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            voList = list.stream().map(converter::convert).collect(Collectors.toList());
        }
        return voList;
    }
}
