package cn.kdan.cloud.pdf.office.payment;

import cn.kdan.cloud.pdf.office.api.account.annotation.EnableAccountApiClient;
import cn.kdan.cloud.pdf.office.api.email.annotation.EnableEmailApiClient;
import cn.kdan.cloud.pdf.office.api.product.annotation.EnableProductApiClient;
import cn.kdan.cloud.pdf.office.common.annotation.*;
import cn.kdan.cloud.pdf.office.common.config.RedissonConfig;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.properties.GooglePayConfig;
import cn.kdan.cloud.pdf.office.payment.properties.PaddleProperties;
import cn.kdan.cloud.pdf.office.payment.properties.PayCenterConfigProperties;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import cn.kdan.cloud.pdf.office.payment.verifier.Verifier;
import lombok.RequiredArgsConstructor;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.netflix.hystrix.EnableHystrix;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Import;
import org.springframework.scheduling.annotation.EnableScheduling;

/**
 * @author ComPDFKit-WPH 2023/1/12
 */

@SpringBootApplication
@EnableDiscoveryClient
@EnableHystrix
@EnableConfigurationProperties({PaypalProperties.class, PaddleProperties.class, AppStoreProperties.class, GooglePayConfig.class, PayCenterConfigProperties.class})
@EnableMybatisPlusConfig
@MapperScan("cn.kdan.cloud.pdf.office.payment.mapper")
@EnableMyRabbitMqHandler
@RequiredArgsConstructor
@EnableCaching
@EnableCommonTools
@EnableProductApiClient
@EnableAccountApiClient
@EnableEmailApiClient
@EnableCommonFeignExceptionHandle
@EnableControllerExceptionHandler
@Import({RedissonConfig.class})
@EnableScheduling
public class PDFOfficePaymentApplication {

    public static void main(String[] args) {
        SpringApplication.run(PDFOfficePaymentApplication.class, args);
    }

    @Bean
    public Verifier getVerifier(PaddleProperties properties) {
        return new Verifier(properties.getPublicKey());
    }


}
