package cn.kdan.cloud.pdf.office.payment.client;

import cn.hutool.core.util.StrUtil;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppStoreReceiptInfo;
import cn.kdan.cloud.pdf.office.api.payment.appstore.AppTransaction;
import cn.kdan.cloud.pdf.office.api.payment.appstore.HistoryResponse;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.GooglePayDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.payment.enums.AppStoreReceiptVerifyStatusEnum;
import cn.kdan.cloud.pdf.office.payment.enums.SubscriptionAckStateEnum;
import cn.kdan.cloud.pdf.office.payment.interceptor.AppStoreClientHttpRequestInterceptor;
import cn.kdan.cloud.pdf.office.payment.interceptor.GoogleClientHttpRequestInterceptor;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.properties.GooglePayConfig;
import cn.kdan.cloud.pdf.office.payment.utils.BearerTokenAuthenticator;
import cn.kdan.cloud.pdf.office.payment.utils.IOUtils;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionPurchaseV3;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.json.gson.GsonFactory;

import com.google.api.services.androidpublisher.AndroidPublisher;
import com.google.api.services.androidpublisher.AndroidPublisherScopes;
import com.google.api.services.androidpublisher.model.SubscriptionPurchaseV2;
import com.google.auth.http.HttpCredentialsAdapter;
import com.google.auth.oauth2.GoogleCredentials;
import com.google.gson.JsonObject;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.*;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @author ComPDFKit-WPH 2023/6/28
 */
@Component
@Slf4j
public class GooglePayClient {

    private final RestTemplate restTemplate = new RestTemplate();


    private AndroidPublisher androidPublisher;// 注入进来咔咔用

    private final RabbitTemplate rabbitTemplate;


    public GooglePayClient(GooglePayConfig properties, GoogleClientHttpRequestInterceptor googleClientHttpRequestInterceptor, RabbitTemplate rabbitTemplate) {
        this.rabbitTemplate = rabbitTemplate;
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        restTemplate.setRequestFactory(factory);
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(@NotNull ClientHttpResponse clientHttpResponse) throws IOException {

            }
        });
    }


    private void getAndroidPublisher() {
        try {
            GoogleCredentials googleCredentials = GoogleCredentials.fromStream(new ClassPathResource("focus-pottery-439503-a3-e3bbeddf9ae0.json").getInputStream())
                .createScoped(AndroidPublisherScopes.ANDROIDPUBLISHER);
            this.androidPublisher = new AndroidPublisher.Builder(
                GoogleNetHttpTransport.newTrustedTransport(),
                GsonFactory.getDefaultInstance(),
                new HttpCredentialsAdapter(googleCredentials)
        ).setApplicationName("PDF Reader Pro").build();
        } catch (Exception e) {
            log.error("[Google] Get android publisher exception: {}", e.getMessage());
            throw new BackendRuntimeException("[Google] Get android publisher exception: " + e.getMessage());
        }
    }

    public SubscriptionPurchaseV3 getSubscription(String packageName, String purchaseToken) {
        try {
        String url = "https://backend.pdfreaderpro.com/system/google/getSubscriptionGoogleOrderV2" +
                "?packageName=" + packageName + "&purchaseToken=" + purchaseToken;

        // 设置请求头
        HttpHeaders headers = new HttpHeaders();
        // 创建 HttpEntity
        HttpEntity<String> entity = new HttpEntity<>(headers);

        // 发起请求

        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);
        SubscriptionPurchaseV3 subscriptionsV2 = JSON.parseObject(response.getBody(),SubscriptionPurchaseV3.class);
        return subscriptionsV2;
        } catch (Exception e) {
            log.error("谷歌订单校验异常,从会员系统获取订阅失败purchaseToken:{}，package:{}",purchaseToken,packageName);
            throw new BackendRuntimeException("[Google] Get google subscriptions v2 order info exception: " + e.getMessage());
        }
    }

    public SubscriptionPurchaseV2 getSubscriptionGoogleOrderV2(String packageName, String purchaseToken){
        getAndroidPublisher();
        SubscriptionPurchaseV2 subscriptionsV2 = null;
        try {
            subscriptionsV2 = androidPublisher.purchases().subscriptionsv2().get(packageName, purchaseToken).execute();
            log.info("SubscriptionsV2 order info: {}", JsonUtils.getJsonString(subscriptionsV2));
        } catch (IOException e) {
            log.error("谷歌订单校验异常,purchaseToken:{}",purchaseToken);
            throw new BackendRuntimeException("[Google] Get google subscriptions v2 order info exception: " + e.getMessage());
        }
        return subscriptionsV2;
    }

    /**
     * 校验订阅订单是否支付成功
     *
     * @param subscriptionsV2
     * @return
     */
    public boolean checkSubscriptionSuccess(SubscriptionPurchaseV3 subscriptionsV2) {
        String acknowledgementState = subscriptionsV2.getAcknowledgementState();
        if (SubscriptionAckStateEnum.ACKNOWLEDGEMENT_STATE_ACKNOWLEDGED.getCode().equals(acknowledgementState)) {
            return true;
        }
        return false;
    }
    public static void main(String[] args) {
//        testID();
        RestTemplate restTemplate = new RestTemplate();
        String url = "https://backend.pdfreaderpro.com/system/google/getSubscriptionGoogleOrderV2" +
                "?packageName=" + "com.pdftechnologies.pdfreaderpro" + "&purchaseToken="
                + "opajmgohldedlcaneafhjibe.AO-J1OxaTTocxdvDeUROv4lNFQXdw7hz9S2EfVkeQZBed4D6IUBmX9elKLQ04_2BbBE0vgaZulHWa2ZtTswBN-UDsFZVrL2EMJknSJUnraA_mPsGrD3Y8sQ";

        // 设置请求头
        HttpHeaders headers = new HttpHeaders();
        // 创建 HttpEntity
        HttpEntity<String> entity = new HttpEntity<>(headers);

        // 发起请求
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, entity, String.class);
        //判断有没有testPurchase 字段
        SubscriptionPurchaseV3 subscriptionsV2 = JSON.parseObject(response.getBody(),SubscriptionPurchaseV3.class);
//        // 创建目标对象
        SubscriptionPurchaseV2 venueV2 = new SubscriptionPurchaseV2();
//
//        // 复制属性
//        BeanUtils.copyProperties(subscriptionsV2, venueV2);
//        System.out.println("123");
//        String Period = "7.days,1.years";
//        //7.days:0,1.years:20,1.years:40
//        List<String> dateAndPriceList = Arrays.asList(Period.split(","));
//        int times =  dateAndPriceList.size() > 1 ? 1 : dateAndPriceList.size()-1;
//        String forPeriod = dateAndPriceList.get(times);;
//        LocalDateTime dateTime = LocalDateTime.now();
//        MyDateUtils.TimeAddition(dateTime, forPeriod);
//        System.out.println(111);
    }

    private static void testID() {
        String appName = "PDF Reader Pro";
        String packageName = "com.pdftechnologies.pdfreaderpro";
        String purchaseToken = "lcjenpgpddjefjihgknnhfgc.AO-J1Ow5p88lKdizid8BDaK_jV3vzjzIZM_qf_S0U9lCNmY6aACQIs_3cgs-ntM6-bC2PNWJdR2YhsN9fWHSu_9BEFhD3-uBJGG39zzgaYz9jimQIcUCvPs" ;
        InputStream content = new IOUtils().getGoogleKey();
        try {
            GoogleCredentials googleCredentials = GoogleCredentials.fromStream(content)
                    .createScoped(AndroidPublisherScopes.ANDROIDPUBLISHER);
           AndroidPublisher androidPublisher = new AndroidPublisher.Builder(
                    GoogleNetHttpTransport.newTrustedTransport(),
                    GsonFactory.getDefaultInstance(),
                    new HttpCredentialsAdapter(googleCredentials)
            ).setApplicationName(appName).build();
            SubscriptionPurchaseV2 subscriptionsV2 = null;
            try {
                subscriptionsV2 = androidPublisher.purchases().subscriptionsv2().get(packageName, purchaseToken).execute();
                log.info("SubscriptionsV2 order info: {}", JsonUtils.getJsonString(subscriptionsV2));
            } catch (IOException e) {
                log.error("谷歌订单校验异常");
                throw new BackendRuntimeException("[Google] Get google subscriptions v2 order info exception: " + e.getMessage());
            }
        } catch (Exception e) {
            log.error("[Google] Get android publisher exception: {}", e.getMessage());
            throw new BackendRuntimeException("[Google] Get android publisher exception: " + e.getMessage());
        }
    }
}
