package cn.kdan.cloud.pdf.office.payment.enums;

import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import org.springframework.util.ObjectUtils;


/**
 * @author tangxiangan
 */

public enum PaypalTransactionStatusEnum {

    /**
     * Enum:	Description
     * COMPLETED	The funds for this captured payment were credited to the payee's PayPal account.
     * DECLINED	The funds could not be captured.
     * PARTIALLY_REFUNDED	An amount less than this captured payment's amount was partially refunded to the payer.
     * PENDING	The funds for this captured payment was not yet credited to the payee's PayPal account. For more information, see status.details.
     * REFUNDED	An amount greater than or equal to this captured payment's amount was refunded to the payer.
     */
    COMPLETED("COMPLETED"),
    // 	订阅已取消。
    DECLINED("DECLINED"),

    //订阅激活
    PARTIALLY_REFUNDED("PARTIALLY_REFUNDED"),

    //订阅扣款成功s
    PENDING("PENDING"),

    // 订阅付款失败。
    REFUNDED("REFUNDED"),
    ;

    private final String value;

    PaypalTransactionStatusEnum(String value) {
        this.value = value;
    }

    public String value() {
        return this.value;
    }

    public static PaypalTransactionStatusEnum getByValue(String value) {
        for (PaypalTransactionStatusEnum paypalWebhookEventEnum : PaypalTransactionStatusEnum.values()) {
            if (ObjectUtils.nullSafeEquals(paypalWebhookEventEnum.value, value)) {
                return paypalWebhookEventEnum;
            }
        }
        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EVENT_IS_CURRENTLY_NOT_SUPPORTED);
    }


}
