package cn.kdan.cloud.pdf.office.payment.enums;

import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import org.springframework.util.ObjectUtils;

/**
 * @author ComPDFKit-WPH 2023/6/7
 * <p>
 * https://developer.paypal.com/api/rest/webhooks/event-names/#subscriptions
 */
public enum PaypalWebhookEventEnum {

    // 商家为商品退款。
    SALE_REFUNDED("PAYMENT.SALE.REFUNDED"),
    // 	订阅已取消。
    BILLING_SUBSCRIPTION_CANCELLED("BILLING.SUBSCRIPTION.CANCELLED"),

    //订阅激活
    PLAN_ACTIVATED("BILLING.SUBSCRIPTION.ACTIVATED"),

    //订阅扣款成功s
    SALE_COMPLETED("PAYMENT.SALE.COMPLETED"),

    // 订阅付款失败。
    BILLING_SUBSCRIPTION_PAYMENT_FAILED("BILLING.SUBSCRIPTION.PAYMENT.FAILED"),
    ;

    private final String value;

    PaypalWebhookEventEnum(String value) {
        this.value = value;
    }

    public String value() {
        return this.value;
    }

    public static PaypalWebhookEventEnum getByValue(String value) {
        for (PaypalWebhookEventEnum paypalWebhookEventEnum : PaypalWebhookEventEnum.values()) {
            if (ObjectUtils.nullSafeEquals(paypalWebhookEventEnum.value, value)) {
                return paypalWebhookEventEnum;
            }
        }
        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_EVENT_IS_CURRENTLY_NOT_SUPPORTED);
    }


}
