package cn.kdan.cloud.pdf.office.payment.utils;

import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.interfaces.ECPrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.time.Instant;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import org.apache.commons.codec.binary.Base64;
/**
 * @author ComPDFKit-WPH 2023/6/29
 */
public class AppStoreTokenUtils {
    public static String getToken() {

        Map<String, Object> header = new HashMap<>();

        header.put("alg", "ES256");
        header.put("kid", "321ee8da056a45e7b963eb519c25c4fc");
        header.put("typ", "JWT");

        Map<String, Object> claim = new HashMap<>();
        claim.put("iss", "321ee8da056a45e7b963eb519c25c4fc");
        claim.put("iat", Math.floor(System.currentTimeMillis() / 1000));
        claim.put("exp", Math.floor(System.currentTimeMillis() / 1000) + 1800);
        claim.put("aud", "appstoreconnect-v1");
        PrivateKey privateKey = getECPrivateKey();
        try {
            JwtBuilder jwtBuilder = Jwts.builder().setHeader(header).setClaims(claim)
                    .signWith(SignatureAlgorithm.ES256, privateKey);

            String token = jwtBuilder.compact();
            return token;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return  "";
    }

    /**
     * 获取PrivateKey对象
     *
     * @return
     */
    private static PrivateKey getECPrivateKey() {
        try {
            PKCS8EncodedKeySpec pkcs8EncodedKeySpec = new PKCS8EncodedKeySpec(
                    Base64.decodeBase64("321ee8da056a45e7b963eb519c25c4fc"));
            KeyFactory keyFactory = KeyFactory.getInstance("EC");
            return keyFactory.generatePrivate(pkcs8EncodedKeySpec);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void main(String[] args) throws NoSuchAlgorithmException, IOException, InvalidKeySpecException {
        String teamId = "YOUR_TEAM_ID";
        String keyId = "YOUR_KEY_ID";
        String privateKeyFilePath = "/path/to/your/private_key.p8";

        String accessToken = generateAccessToken(teamId, keyId, privateKeyFilePath);
        System.out.println("Generated Access Token: " + accessToken);
    }

    public static String generateAccessToken(String teamId, String keyId, String privateKeyFilePath)
            throws NoSuchAlgorithmException, InvalidKeySpecException, IOException {

        // 从私钥文件加载私钥
        byte[] privateKeyBytes = Files.readAllBytes(Paths.get(privateKeyFilePath));
        PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(privateKeyBytes);
        KeyFactory keyFactory = KeyFactory.getInstance("EC");
        ECPrivateKey privateKey = (ECPrivateKey) keyFactory.generatePrivate(keySpec);

        // 创建JWT Builder
        JwtBuilder jwtBuilder = Jwts.builder();

        // 设置Header
        jwtBuilder.setHeaderParam("kid", keyId);
        jwtBuilder.setHeaderParam("typ", "JWT");

        // 设置Payload
        jwtBuilder.setIssuer(teamId);
        jwtBuilder.setIssuedAt(Date.from(Instant.now()));
        jwtBuilder.setExpiration(Date.from(Instant.now().plusSeconds(60 * 10))); // 设置10分钟过期时间

        // 生成Access Token
        String accessToken = jwtBuilder.signWith(SignatureAlgorithm.ES256, privateKey).compact();

        return accessToken;
    }


}
