package cn.kdan.cloud.pdf.office.payment;

import cn.kdan.cloud.pdf.office.api.payment.bo.GeneratePayLinkPaddleBO;
import cn.kdan.cloud.pdf.office.api.payment.bo.OrderSucceededBO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.paddle.UpdateSubscription;
import cn.kdan.cloud.pdf.office.api.payment.paddle.UpdateSubscriptionResult;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import org.junit.jupiter.api.Test;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.math.BigDecimal;

/**
 * @author ComPDFKit-WPH 2023/1/16
 */
@SpringBootTest
public class PaddleTest {

    @Autowired
    private PaddleClient paddleClient;
    @Autowired
    private OrderService ordersService;

    @Test
    public void createS() {
        GeneratePayLinkPaddleBO generatePayLinkPaddleBO = new GeneratePayLinkPaddleBO();
        generatePayLinkPaddleBO.setProductId("38247");
        generatePayLinkPaddleBO.setPrices(new String[]{"USD:" + "1.00"});
        System.out.println(paddleClient.createSubscriptions(generatePayLinkPaddleBO));
    }

    @Test
    public void createOrder() {
        CreateOrderDTO createOrderDTO = CreateOrderDTO.builder()
                .appId("12312331223")
                .productId("1")
                .userId("2312232221")
                .paymentMethod(PaymentMethodEnum.PADDLE)
                .build();
        System.out.println(ordersService.createOrder(createOrderDTO));
    }


    @Test
    public void testUp() {
        UpdateSubscription updateSubscription = new UpdateSubscription();
        updateSubscription.setThirdSubscriptionId("420996");
        updateSubscription.setRecurringPrice("120.00");
        updateSubscription.setCurrency("USD");
        updateSubscription.setQuantity(1);
        UpdateSubscriptionResult updateSubscriptionResult = paddleClient.updateSubscription(updateSubscription);
    }

    @Test
    public void testActiveSub() {
       paddleClient.getSubscriptionInfo("533832");
    }

    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Test
    public void getSInfo() {
        int ttl = 60 * 1000;
        // 订阅过期
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE, RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_ROUTING_KEY,
                "321",
                (msg -> {
                    //发送消息 并设置delayedTime
                    msg.getMessageProperties().setHeader("x-delay", ttl + "");
                    return msg;
                }));
    }

    @Test
    public void testOrderS(){
        OrderSucceededBO orderSucceeded = OrderSucceededBO.builder()
                .thirdTradeNo("2000000357671061")
                .price(BigDecimal.ONE)
                .subscriptionId("123")
                .thirdOrderNo("2000000360970615")
                .userId("018891d5-45e8-4566-8c07-5c684b9cbdd6")
                .build();
        ordersService.updateOrderSucceeded(orderSucceeded);
    }

    public static void main(String[] args) {
        System.out.println(Integer.MAX_VALUE / 1000 / 60 / 60 / 24);
    }
//    }
//
//    public static void main(String[] args) {
//        GeneratePayLinkPaddleBO generatePayLinkPaddleBO = new GeneratePayLinkPaddleBO();
//        generatePayLinkPaddleBO.setDiscountable(true);
//        System.out.println(generatePayLinkPaddleBO.getDiscountable());
//
//    }

}
