package cn.kdan.cloud.pdf.office.website.client;

import cn.kdan.cloud.pdf.office.website.dto.ai.ComPdfKitResult;
import cn.kdan.cloud.pdf.office.website.dto.ai.FileTranslateHandleVO;
import cn.kdan.cloud.pdf.office.website.dto.ai.FileUploadVo;
import cn.kdan.cloud.pdf.office.website.dto.ai.TextVO;
import cn.kdan.cloud.pdf.office.website.properties.AIProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.*;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;

@Component
public class AIApiClient {
    private final RestTemplate restTemplate;
    private final AIProperties aiProperties;
    private final String baseUrl;
    private final String translatedUrl;

    public AIApiClient(AIProperties aiProperties) {
        this.aiProperties = aiProperties;
        this.restTemplate = new RestTemplate();
        this.baseUrl = this.aiProperties.getBaseUrl();
        this.translatedUrl = this.aiProperties.getTranslatedUrl();
    }

    public String correctTypos(String content) {
        String url = baseUrl + "/api/correct-typos";
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("User-Agent", "Apifox/1.0.0 (https://apifox.com)");

        String json = String.format("{\"content\":\"%s\",\"project_id\":%d}", content, 4);
        HttpEntity<String> entity = new HttpEntity<>(json, headers);

        return restTemplate.postForObject(url, entity, String.class);
    }

    public String extractFileSummary(MultipartFile file) {
        try {
            String url = baseUrl + "/api/extract-file-summary";
            HttpHeaders headers = new HttpHeaders();
            headers.set("User-Agent", "Apifox/1.0.0 (https://apifox.com)");
            File tempFile = null;
            tempFile = File.createTempFile("tempFile", file.getOriginalFilename());
            tempFile.deleteOnExit();
            file.transferTo(tempFile);
            MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
            body.add("file", new FileSystemResource(tempFile));
            body.add("project_id", 4);

            HttpEntity<MultiValueMap<String, Object>> entity = new HttpEntity<>(body, headers);

            return restTemplate.postForObject(url, entity, String.class);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public String rewrite(String content) {
        String url = baseUrl + "/api/rewrite";
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.set("User-Agent", "Apifox/1.0.0 (https://apifox.com)");

        String json = String.format("{\"content\":\"%s\",\"project_id\":%d}", content, 4);
        HttpEntity<String> entity = new HttpEntity<>(json, headers);

        return restTemplate.postForObject(url, entity, String.class);
    }

    public ComPdfKitResult<FileUploadVo> fileUpload(MultipartFile file, int calculate) {
        try {
            File tempFile = File.createTempFile("tempFile", file.getOriginalFilename());
            tempFile.deleteOnExit();
            file.transferTo(tempFile);
            String url = translatedUrl + "/v1/translate/fileUpload";
            HttpHeaders headers = new HttpHeaders();
            headers.set("User-Agent", "Apifox/1.0.0 (https://apifox.com)");

            MultiValueMap<String, Object> body = new LinkedMultiValueMap<>();
            body.add("file", new FileSystemResource(tempFile)); // 这里使用FileSystemResource
            body.add("projectId", 4);
            body.add("calculate", calculate);

            HttpEntity<MultiValueMap<String, Object>> entity = new HttpEntity<>(body, headers);

            ParameterizedTypeReference<ComPdfKitResult<FileUploadVo>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<FileUploadVo>>() {};
            ResponseEntity<ComPdfKitResult<FileUploadVo>> response = restTemplate.exchange(
                    url,
                    HttpMethod.POST,
                    entity,
                    typeRef
            );
            return response.getBody();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public ComPdfKitResult<FileTranslateHandleVO> fileTranslate(String fileKey, String from, String to, int charCount, String fileId) {
        String url = translatedUrl + "/v1/translate/fileTranslateHandle";
        HttpHeaders headers = new HttpHeaders();
        headers.set("User-Agent", "Apifox/1.0.0 (https://apifox.com)");
        headers.set("Content-Type", "application/json");

        String jsonInputString = String.format(
                "{\"fileKey\":\"%s\",\"from\":\"%s\",\"to\":\"%s\",\"projectId\":%d,\"charCount\":%d,\"fileId\":\"%s\"}",
                fileKey, from, to, 4,charCount,fileId
        );

        HttpEntity<String> entity = new HttpEntity<>(jsonInputString, headers);
        ParameterizedTypeReference<ComPdfKitResult<FileTranslateHandleVO>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<FileTranslateHandleVO>>() {};
        ResponseEntity<ComPdfKitResult<FileTranslateHandleVO>> response = restTemplate.exchange(
                url,
                HttpMethod.POST,
                entity,
                typeRef
        );
        return response.getBody();
    }

    public ComPdfKitResult<TextVO> textTranslate(String q, String from, String to) {
        String url = translatedUrl + "/v1/translate/textTrans";
        HttpHeaders headers = new HttpHeaders();
        headers.set("Content-Type", "application/json");

        String jsonInputString = String.format(
                "{\"q\":\"%s\",\"from\":\"%s\",\"to\":\"%s\",\"projectId\":%d}",
                q, from, to,  4
        );

        HttpEntity<String> entity = new HttpEntity<>(jsonInputString, headers);
        ParameterizedTypeReference<ComPdfKitResult<TextVO>> typeRef = new ParameterizedTypeReference<ComPdfKitResult<TextVO>>() {};
        ResponseEntity<ComPdfKitResult<TextVO>> response = restTemplate.exchange(
                url,
                HttpMethod.POST,
                entity,
                typeRef
        );
        return response.getBody();
    }


}
