package cn.kdan.cloud.pdf.office.website.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.annotation.Order;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.token.RemoteTokenServices;
import org.springframework.security.oauth2.provider.token.ResourceServerTokenServices;
import org.springframework.web.client.RestTemplate;


/**
 * 资源提供端的配置
 */
@Configuration
@EnableResourceServer //开启资源提供服务的配置  是默认情况下spring security oauth2的http配置会被WebSecurityConfigurerAdapter的配置覆盖
@Order(3)
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

    @Value("${httpMatchers.web}")
    private String httpWebMatchers;
    @Value("${security.oauth2.client.client-id}")
    private String clientId;
    @Value("${security.oauth2.client.client-secret}")
    private String clientSecret;
    @Value("${security.oauth2.resource.token-info-uri}")
    private String tokenInfoUri;
    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.IF_REQUIRED);
        http.authorizeRequests().anyRequest().authenticated();
    }

    @Bean
    public ResourceServerTokenServices tokenService() {
        // 是用于向远程认证服务器验证token,同时获取token对应的用户的信息
        CustomRemoteTokenServices remoteTokenServices = new CustomRemoteTokenServices();
        // 自定义token转换器
        remoteTokenServices.setCheckTokenEndpointUrl(tokenInfoUri);
        remoteTokenServices.setClientId(clientId);
        remoteTokenServices.setClientSecret(clientSecret);
        remoteTokenServices.setRestTemplate(new RestTemplate());
        return remoteTokenServices;
        //使用远程服务请求授权服务器校验token,必须指定校验token 的url、client_id，client_secret
    }

}
