package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.AIApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.feign.UserSubscriptionInfoApi;
import cn.kdan.cloud.pdf.office.api.account.vo.AiRecord;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.enums.AIActionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserSubscriptionStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.website.dto.ai.*;
import cn.kdan.cloud.pdf.office.website.service.impl.AIServiceImpl;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.security.Principal;
import java.security.Security;
import java.util.*;

/**
 * 翻译模块
 */
@RestController
@RequestMapping("/v1/translate")
@RequiredArgsConstructor
public class TranslateController {
    @Autowired
    private UserSubscriptionInfoApi userSubscriptionInfoApi;
    @Autowired
    private AIServiceImpl aiService;
    @Autowired
    private AIApi aiApi;
    @Autowired
    private UserApi userApi;


    /**
     * 文件上传
     * @param file
     * @param calculate
     * @param principal
     * @return
     */
    @Transactional
    @PostMapping("/fileUpload")
    public Object fileUpload (@RequestParam MultipartFile file, @RequestParam Integer calculate, Principal principal){

        try {
            TranslateParam param = new TranslateParam();
            param.setFile(file);
            param.setCalculate(calculate);
            String userId=SecurityUtils.getUserId();
            TranslateResult<FileUploadVo> result =  aiService.fileUpload(param);
            FileUploadVo vo = result.getData();
            aiRecord(vo.getFileKey(), userId, result,0,AIActionEnum.UPLOAD_TRANSLATION_FILE,vo.getFileId(),vo.getCharCount());
            Integer cost;
            if (vo.getCharCount() < 10000) {
                cost = 2; // 不满1000字符的费用为1
            } else {
                cost = (int) Math.ceil(vo.getCharCount() / 10000.0 )*2;
            }
            UserSubscriptionInfoVO userSubscriptionInfoVO = getAiSubscriptionInfo(principal);
            if(!checkPoint(userSubscriptionInfoVO, cost)){
                TranslateResult<Map<String,Integer>> result1 = new TranslateResult<Map<String,Integer>>();
                result1.setCode("500");
                result1.setMessage("run out");
                Map<String,Integer> map1 = new HashMap<>();
                map1.put("credit",cost);
                result1.setData(map1);
                return result1;
            }
            vo.setCredit(cost);
            result.setData(vo);
            return result;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }

    private void aiRecord(String fileKey, String userId, Object object, Integer point,AIActionEnum actionEnum,String fileId,Integer charCount) {
        AiRecord aiRecord = new AiRecord();
        aiRecord.setFunction(actionEnum.getCode());
        aiRecord.setId(CommonUtils.generateId());
        if(!StringUtils.isEmpty(fileId)){
            aiRecord.setFileId(fileId);
        }
        if(!StringUtils.isEmpty(charCount)){
            aiRecord.setCharCount(charCount);
        }
        if(!StringUtils.isEmpty(fileKey)){
            aiRecord.setFileKey(fileKey);
        }
        aiRecord.setUserId(userId);
        aiRecord.setResult(JsonUtils.getJsonString(object));
        aiRecord.setCredit(point);
        aiRecord.setCreatedAt(new Date());
        aiApi.insert(aiRecord);
    }

    /**
     * 文件翻译
     * @param param param
     * @return Object
     */
    @Transactional
    @RequestMapping("/fileTranslateHandle")
    public Object fileTranslateHandle (@RequestBody TranslateParam param,Principal principal){
        try {
            UserSubscriptionInfoVO vo = getAiSubscriptionInfo(principal);
            AiRecord record = aiApi.getByFileKey(param.getFileKey()).getResult();
            String fileId = record.getFileId();
            Integer charCount = record.getCharCount();
            param.setFileId(fileId);
            param.setCharCount(charCount);
            Integer cost;
            if (charCount < 10000) {
                cost = 2; // 不满1000字符的费用为1
            } else {
                cost = (int) Math.ceil(charCount / 1000.0 )*2;
            }
            if(!checkPoint(vo, cost)){
                TranslateResult<Map<String,Integer>> result = new TranslateResult<Map<String,Integer>>();
                result.setCode("500");
                result.setMessage("run out");
                Map<String,Integer> map = new HashMap<>();
                map.put("credit",cost);
                result.setData(map);
                return result;
            }
            Object object = aiService.fileTranslateHandle(param);
            decPoint(vo, cost);
            aiRecord(null, vo.getUserId(), object,cost,AIActionEnum.FILE_TRANSLATION,null,null);
            return object;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }

    /**
     * 文本翻译
     * @param param param
     * @return
     */
    @Transactional
    @RequestMapping("/textTrans")
    public Object textTrans (@RequestBody TranslateParam param,Principal principal){
        try {
            UserSubscriptionInfoVO vo = getAiSubscriptionInfo(principal);
            if(!checkPoint(vo, 1)){
                TranslateResult<Map<String,Integer>> result = new TranslateResult<Map<String,Integer>>();
                result.setCode("500");
                result.setMessage("run out");
                Map<String,Integer> map = new HashMap<>();
                map.put("credit",1);
                return result;
            }
            TranslateResult<TextVO> object = aiService.textTrans(param);
            decPoint(vo, 1);
            aiRecord(null, vo.getUserId(), object,1,AIActionEnum.TEXT_TRANSLATION,null,null);
            return object;
        } catch (Exception e) {
            TranslateResult<String> result = new TranslateResult<String>();
            result.setCode("500");
            result.setMessage("run out");
            return result;
        }
    }
    public UserSubscriptionInfoVO getAiSubscriptionInfo(Principal principal) {
        String userId = SecurityUtils.getUserId();
        UserInfoVO infoVO =  userApi.getMemberInfoById(userId).getResult();
        List<UserSubscriptionInfoVO> voList =  userSubscriptionInfoApi.getByUserIdAndAppId(userId,"1").getResult();
        Optional<UserSubscriptionInfoVO> activeUserSubscriptionInfoVO = voList.stream()
                .filter(vo -> {
                    // Check if 'isAi' is not empty
                    return !ObjectUtils.isEmpty(vo.getIsAi()) &&
                            vo.getStatus().equals(PDFOfficeUserSubscriptionStatusEnum.SUBSCRIPTION_IN_PROGRESS.value());
                })
                .findFirst(); // Get the first matching VO
        UserSubscriptionInfoVO userSubscriptionInfoVO = activeUserSubscriptionInfoVO.get();
        return userSubscriptionInfoVO;
    }

    public boolean checkPoint(UserSubscriptionInfoVO infoVO, Integer point){
        //如果point不够返回false
        if(infoVO.getPoint()<point){
            return false;
        }
        return true;
    }

    public void decPoint(UserSubscriptionInfoVO infoVO,Integer point){
        UserSubscriptionInfo userSubscriptionInfo = new UserSubscriptionInfo();
        userSubscriptionInfo.setId(infoVO.getId());
        userSubscriptionInfo.setPoint(infoVO.getPoint() - point);
        userSubscriptionInfoApi.update(userSubscriptionInfo);
    }


}
