package cn.kdan.cloud.pdf.office.product.service;

import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
public interface ProductService extends IService<Product> {

    /**
     * 列表查询
     * @param query 查询参数
     * @return 列表数据
     */
    IPage<ProductVO> page(ProductListDTO query);

    /**
     * 获取产品价格
     *
     * @param productId 产品id
     * @param userId userId
     * @return 价格
     */
    ProductPriceVo getProductPrice(String productId,String userId);

    /**
     * 通过id 获取产品信息
     *
     * @param productId id
     * @return Product
     */
    ProductVO getProduct(String productId);

    ProductVO getProductByAppIdAndModel(String appId, Integer model);

    ProductVO getProductByCode(String code);

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    List<ProductVO> getNewUserProduct(String appId);

    /**
     * 获取产品（带第三方支付id）
     * @param productId
     * @param payment
     * @param isDiscount
     * @return
     */
    ProductVO getProductForPay(String productId, Integer payment, Integer isDiscount);

    ProductVO getProductByThirdPayId(String thirdPayId);

    List<ListingProductVO> getListingProducts(Integer isEducation, String platformId);

    List<ListingProductVO> getProductsByIds(List<String> productIds);

    /**
     * 验证用户邮箱教育优惠资格
     * @param email 用户邮箱
     * @return
     */
    Integer checkEmail(String email);

//    ListingProductVO getProductPriceForBuy(String productId, String userId, Integer isEducation);

    /**
     * 批量获取产品价格
     * @return
     */
    BigDecimal getBatchProductPrice(String productId, Integer num);

    /**
     * 验证优惠券、奖品是否可用，返回优惠后价格
     */
    ListingProductVO checkCoupon(String productId, String userId, String code);

    Boolean updateCouponState(String code, Integer state, String orderId);

    Boolean refundCouponByOrderId(String orderId);
}
