package cn.kdan.cloud.pdf.office.product.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.utils.SubscriptionUtil;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.api.product.dto.ProductListDTO;
import cn.kdan.cloud.pdf.office.api.product.vo.ActivityInfoVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.product.entity.Prize;
import cn.kdan.cloud.pdf.office.product.entity.Product;
import cn.kdan.cloud.pdf.office.product.entity.RProductPay;
import cn.kdan.cloud.pdf.office.product.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.product.mapper.ProductMapper;
import cn.kdan.cloud.pdf.office.product.service.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author ComPDFKit-WPH 2023-01-28
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class ProductServiceImpl extends ServiceImpl<ProductMapper, Product> implements ProductService {

    private final ActivityInformationService activityService;

    private final UserApi userAPi;

//    private final OrderApi orderApi;

    private final RProductPayService rProductPayService;

    private final EduEmailsService eduEmailsService;

    private final PrizeService prizeService;

    @Value("${blackFiveProduct:advanced-permanent-blackFive,advanced-annual-subscription-blackFive}")
    private List<String> blackFiveProduct;



    @Override
    public IPage<ProductVO> page(ProductListDTO query) {
        Page<Product> productPage = this.page(new Page<>(query.getPage(), query.getSize()), new LambdaQueryWrapper<Product>()
                .eq(Product::getAppId, query.getAppId()));
        List<ProductVO> productVOS = JsonUtils.jsonStringToList(JsonUtils.getJsonString(productPage.getRecords()), ProductVO.class);
        productVOS.forEach(productVO -> {
            if (!ObjectUtils.isEmpty(productVO.getActivityId())) {
                productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
            }
        });
        Page<ProductVO> page = new Page<>(query.getPage(), query.getSize());
        page.setRecords(productVOS);
        return page;
    }

    @Override
    public ProductPriceVo getProductPrice(String productId, String userId) {
        Product product = this.baseMapper.selectOne(new LambdaQueryWrapper<Product>()
                .select(Product::getPrice, Product::getCnyPrice, Product::getPaddleProductId, Product::getActivityId)
                .eq(Product::getId, productId));
        ProductPriceVo productPriceVo = new ProductPriceVo();
        productPriceVo.setPaddleProductId(product.getPaddleProductId());
        // 判断当前产品是否有活动 获取活动的最终价格
        if (!ObjectUtils.isEmpty(product.getActivityId())) {
            ActivityInfoVO activityInfo = activityService.getActivityInfoByActivityId(product.getActivityId());
            // 判断是否是新用户专用
            List<UserSubscriptionInfoVO> subscriptionInfoList = userAPi.getInfoById(userId).getResult().getSubscriptionInfoList();
            if (activityInfo.getActivityType() == 1) {
                UserSubscriptionInfoVO userSubscriptionInfoVO = SubscriptionUtil.getUserSubscriptionInfoByPlatform(subscriptionInfoList, product.getPlatform());
                if (ObjectUtils.isEmpty(userSubscriptionInfoVO)) {
                    productPriceVo.setPrice(activityInfo.getFinalPrice());
                    productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
                } else {
                    throw new BackendRuntimeException(ErrorMessage.USER_IS_NOT_NEW_USER);
                }
            } else {
                productPriceVo.setPrice(activityInfo.getFinalPrice());
                productPriceVo.setCnyPrice(activityInfo.getFinalCnyPrice());
            }
        }
        return productPriceVo;
    }

    @Override
    public ProductVO getProduct(String productId) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        return productVO;
    }

    @Override
    public ProductVO getProductByAppIdAndModel(String appId, Integer model) {
        List<Product> list = this.baseMapper.selectList(new LambdaQueryWrapper<Product>().eq(Product::getAppId, appId).eq(Product::getPlatform, model));
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(list.get(0),productVO);
        return productVO;
    }

    @Override
    public ProductVO getProductByCode(String code) {
        List<Product> list = this.baseMapper.selectList(new LambdaQueryWrapper<Product>().eq(Product::getCode, code));
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(list.get(0),productVO);
        return productVO;
    }

    @Override
    public List<ProductVO> getNewUserProduct(String appId) {
        List<ProductVO> productVOS = this.baseMapper.selectNewUserProductByAppId(appId);
        for (ProductVO productVO : productVOS) {
            productVO.setActivityInfo(activityService.getActivityInfoByActivityId(productVO.getActivityId()));
        }
        return productVOS;
    }

    @Override
    public ProductVO getProductForPay(String productId, Integer payment, Integer isDiscount) {
        ProductVO productVO = new ProductVO();
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Product>().eq(Product::getId, productId)), productVO);
        RProductPay productPay = new RProductPay();
        productPay.setProductId(productId);
        productPay.setPayment(payment);
        productPay.setIsDiscount(isDiscount);
        List<RProductPay>  list = rProductPayService.list(productPay);
        productVO.setThirdPayId(list.get(0).getThirdPayId());
        return productVO;
    }

    @Override
    public ProductVO getProductByThirdPayId(String thirdPayId) {
        ProductVO productVO = this.baseMapper.getProductByThirdPayId(thirdPayId);
        if (ObjectUtils.isEmpty(productVO)) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_THIRD_PAY_ID_ILLEGAL);
        }
        return productVO;
    }

    @Override
    public List<ListingProductVO> getListingProducts(Integer isEducation, String platformId) {
        // 获取上架中的产品，默认不查教育商品
        List<Integer> discountTypeList = new ArrayList<>();
        discountTypeList.add(0);
        if (isEducation == 1) {
            // 教育
            discountTypeList.add(1);
        }
        // 黑五
        discountTypeList.add(2);
        List<ListingProductVO> listingProducts = this.baseMapper.getProductsByPlatformIdAndDiscountType(platformId, discountTypeList);
        /*if (null != isEducation && isEducation == 1) {
            // 返回原价和教育优惠价格
            listingProducts.forEach(listingProductVO -> {
                ProductEducationPriceEnum byPrice = ProductEducationPriceEnum.getByPrice(listingProductVO.getPrice());
                ProductEducationPriceEnum cnyPrice = ProductEducationPriceEnum.getByPrice(listingProductVO.getCnyPrice());
                if (null != byPrice && null != cnyPrice) {
                    listingProductVO.setDisplayPrice(byPrice.getDisplayPrice());
                    listingProductVO.setDisplayCnyPrice(cnyPrice.getDisplayPrice());
                } else {
                    log.error("产品价格设置错误，id：{}", listingProductVO.getId());
                }
            });
        }*/

        if (null != isEducation && isEducation == 2) {
            // 获取商品code在blackFiveProduct里的产品
            this.baseMapper.selectList(new LambdaQueryWrapper<Product>().in(Product::getCode, blackFiveProduct)).forEach(product -> {
                ListingProductVO listingProductVO = new ListingProductVO();
                BeanUtils.copyProperties(product, listingProductVO);

                String productCode = listingProductVO.getCode();
                if (productCode.endsWith("blackFive")) {
                    // 获取截取后的 code
                    String trimmedCode = productCode.replace("-blackFive", "");
                    // 替换 listingProducts 中对应的产品
                    listingProducts.removeIf(existingProduct -> existingProduct.getCode().equals(trimmedCode));
                    listingProducts.add(listingProductVO);
                } else {
                    listingProducts.add(listingProductVO);
                }
            });
        }
        return listingProducts;
    }


    @Override
    public List<ListingProductVO> getProductsByIds(List<String> productIds) {
        return this.baseMapper.getProductsByIds(productIds);
    }

    /**
     * 验证用户邮箱教育优惠资格
     * @param email 用户邮箱
     * @return
     */
    @Override
    public Integer checkEmail(String email) {
        return eduEmailsService.checkEmail(email);
    }

    /*public ListingProductVO getProductPriceForBuy(String productId, String userId, Integer isEducation) {
        // 查询购买的商品信息
        List<ListingProductVO> productVOs = this.baseMapper.getProductsWithPlatformId(null, Collections.singletonList(productId));
        ListingProductVO listingProductVO = productVOs.get(0);
        // 查询用户+所有订单
        if (StringUtils.isEmpty(userId)) {
            return listingProductVO;
        }
        UserVO userVO = userAPi.getById(userId).getResult();
        if (null == userVO) {
            log.error("用户不存在，id：{}", userId);
            return listingProductVO;
        }
        List<OrdersVO> orders = orderApi.getOrderByUserId(userId).getResult();
        if (null != isEducation && isEducation == 1) {
            // 判断用户教育优惠资格
            // 根据用户邮箱
            Integer checkEmail = eduEmailsService.checkEmail(userVO.getEmail());
            if (checkEmail == 0) {
                throw new BackendRuntimeException(ErrorMessage.USER_IS_NOT_EDUCATION_USER);
            }
            // 查询用户所有订单。判断近一年有没有教育优惠订单
            if (orders.stream().anyMatch(ordersVO -> ordersVO.getIsEducation() == 1 && ordersVO.getCreatedAt().isAfter(LocalDateTime.now().minusYears(1)))) {
                throw new BackendRuntimeException(ErrorMessage.USER_HAS_EDUCATION);
            }
            // 校验教育用户资格通过
            listingProductVO.setDisplayPrice(ProductEducationPriceEnum.getByPrice(listingProductVO.getPrice()).getDisplayPrice());
            // 没有购买过的用户直接返回
            if (CollectionUtils.isEmpty(orders)) {
                return listingProductVO;
            }
            // 获取已购买的订单商品及会员权益
            List<String> orderProductIds = orders.stream().map(OrdersVO::getProductId).collect(Collectors.toList());
            List<ListingProductVO> orderProducts = this.baseMapper.getProductsWithPlatformId(null, orderProductIds);
            if (CollectionUtils.isEmpty(orderProducts)) {
                return listingProductVO;
            }
            // 筛选订单购买的产品中有没有vip订阅类型
            boolean hasVip = false;
            boolean hasSVip = false;
            boolean hasPermanent = false;
            for (ListingProductVO orderProduct : orderProducts) {
                if (orderProduct.getPaymentModel() == 1) {
                    hasPermanent = true;
                } else {
                    if (orderProduct.getLevels().contains("2")) {
                        hasVip = true;
                    } else if (orderProduct.getLevels().contains("3")) {
                        hasSVip = true;
                    }
                }
            }
            // 购买svip，判断是否买断升级订阅（买且只买了永久版，购买高级版订阅）
            if (listingProductVO.getLevels().contains("3")) {
                // vip无法购买svip
                if (hasVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_PREMIUM);
                }
                // svip原价提前续费
                if (hasSVip) {
                    return listingProductVO;
                }
                // 买断升级订阅价格
                if (hasPermanent) {
                    listingProductVO.setUpgradePrice(new BigDecimal("9.99"));
                    return listingProductVO;
                }
            }
            // 购买vip
            if (listingProductVO.getLevels().contains("2")) {
                // vip提前续费
                if (hasVip) {
                    return listingProductVO;
                }
                // svip无法购买vip
                if (hasSVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_VIP);
                }
                // 买断无法购买vip
                if (hasPermanent) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_UPGRADE_VIP);
                }
            }
            return listingProductVO;
        } else {
            // 没有购买过的用户直接返回
            if (CollectionUtils.isEmpty(orders)) {
                return listingProductVO;
            }
            // 获取已购买的订单商品及会员权益
            List<String> orderProductIds = orders.stream().map(OrdersVO::getProductId).collect(Collectors.toList());
            List<ListingProductVO> orderProducts = this.baseMapper.getProductsWithPlatformId(null, orderProductIds);
            if (CollectionUtils.isEmpty(orderProducts)) {
                return listingProductVO;
            }
            // 筛选订单购买的产品中有没有vip订阅类型
            boolean hasVip = false;
            boolean hasSVip = false;
            boolean hasPermanent = false;
            for (ListingProductVO orderProduct : orderProducts) {
                if (orderProduct.getPaymentModel() == 1) {
                    hasPermanent = true;
                } else {
                    if (orderProduct.getLevels().contains("2")) {
                        hasVip = true;
                    } else if (orderProduct.getLevels().contains("3")) {
                        hasSVip = true;
                    }
                }
            }
            // 购买svip，判断是否买断升级订阅（买且只买了永久版，购买高级版订阅）
            if (listingProductVO.getLevels().contains("3")) {
                // vip无法购买svip
                if (hasVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_PREMIUM);
                }
                // svip原价提前续费
                if (hasSVip) {
                    return listingProductVO;
                }
                // 买断升级订阅价格
                if (hasPermanent) {
                    listingProductVO.setUpgradePrice(new BigDecimal("19.99"));
                    return listingProductVO;
                }
            }
            // 购买vip
            if (listingProductVO.getLevels().contains("2")) {
                // vip原价提前续费
                if (hasVip) {
                    return listingProductVO;
                }
                // svip无法购买vip
                if (hasSVip) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_BUY_VIP);
                }
                // 买断无法购买vip
                if (hasPermanent) {
                    throw new BackendRuntimeException(ErrorMessage.USER_CANNOT_UPGRADE_VIP);
                }
            }
            return listingProductVO;
        }
    }*/

    /**
     * 批量获取产品价格
     * @return
     */
    @Override
    public BigDecimal getBatchProductPrice(String productId, Integer num) {
        Product productVO = baseMapper.selectById(productId);
        if (productVO == null) {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        // 2-5	10% OFF	$107.99/个
        //6-20	20% OFF	$95.99/个
        //21-50	30% OFF	$83.99/个
        //50+	联系我们获取最新折扣
        if (num <= 1) {
            return productVO.getPrice();
        } else if (num <= 5) {
            return new BigDecimal("107.99").multiply(new BigDecimal(num));
        } else if (num <= 20) {
            return new BigDecimal("95.99").multiply(new BigDecimal(num));
        } else if (num <= 50) {
            return new BigDecimal("83.99").multiply(new BigDecimal(num));
        } else {
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CONTACT_US);
        }
    }

    @Override
    public ListingProductVO checkCoupon(String productId, String userId, String code) {
        Prize prize = prizeService.getOne(new LambdaQueryWrapper<Prize>()
                .eq(Prize::getCode, code).eq(Prize::getValidFlag, 1));
        if (prize == null) {
            log.error("优惠券不存在:{}", code);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
        }
        if (prize.getType() != 1) {
            log.error("优惠券类型错误:{}", code);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_TYPE_ERROR);
        }
        if (StringUtils.isNotEmpty(prize.getProductId()) && !prize.getProductId().contains(productId)) {
            log.error("优惠券关联产品错误，productId:{}", productId);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
        }
        if (prize.getValidFlag() != 1) {
            log.error("优惠券不可用:{}", code);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
        }
        if (prize.getEndDate().before(new Date()) || prize.getStartDate().after(new Date())) {
            log.error("优惠券时间不可用:{}", code);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
        }
        List<ListingProductVO> productVOs = this.baseMapper.getProductsByIds(Collections.singletonList(productId));
        ListingProductVO listingProductVO = productVOs.get(0);
        if(ObjectUtils.isNotEmpty(listingProductVO.getIsAi())&&listingProductVO.getIsAi().equals(1)){
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
        }
        if(ObjectUtils.isNotEmpty(listingProductVO.getDisplayPrice())){
            if (prize.getActivityType().equals("1")){
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            listingProductVO.setDisplayPrice(listingProductVO.getDisplayPrice().subtract(prize.getUsdPrice()));
            listingProductVO.setCnyDisplayPrice(listingProductVO.getCnyDisplayPrice().subtract(prize.getCnyPrice()));
        }else{
            if ("1".equals(prize.getActivityType())) {
                if (listingProductVO.getDiscountType()==1) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
                }
                if (listingProductVO.getPrice().compareTo(BigDecimal.valueOf(79.99)) < 0) {
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_INVALID);
                }
            }
            listingProductVO.setDisplayPrice(listingProductVO.getPrice().subtract(prize.getUsdPrice()));
            listingProductVO.setCnyDisplayPrice(listingProductVO.getCnyPrice().subtract(prize.getCnyPrice()));
        }
        return listingProductVO;
    }

    @Override
    public Boolean updateCouponState(String code, Integer state, String orderId) {
        return prizeService.update(new Prize(), new LambdaUpdateWrapper<Prize>()
                .set(Prize::getValidFlag, state)
                .set(Prize::getOrderId, orderId)
                .eq(Prize::getCode, code));
    }

    @Override
    public Boolean refundCouponByOrderId(String orderId) {
        return prizeService.update(new Prize(), new LambdaUpdateWrapper<Prize>()
                .set(Prize::getValidFlag, 1).eq(Prize::getOrderId, orderId));
    }
}
