package cn.kdan.cloud.pdf.office.sso.service.impl;

import cn.kdan.cloud.pdf.office.api.account.feign.OauthClientDetailsApi;
import cn.kdan.cloud.pdf.office.api.account.vo.OauthClientDetails;
import cn.kdan.cloud.pdf.office.sso.constant.AuthConstant;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.oauth2.provider.*;
import org.springframework.security.oauth2.provider.client.BaseClientDetails;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

public class CustomClientDetailsServiceImpl implements ClientDetailsService , ClientRegistrationService {

    private Logger logger = LoggerFactory.getLogger(CustomClientDetailsServiceImpl.class);

    private OauthClientDetailsApi oauthClientDetailsApi;

    public CustomClientDetailsServiceImpl(OauthClientDetailsApi oauthClientDetailsApi){
        this.oauthClientDetailsApi = oauthClientDetailsApi;
    }

    @Override
    public ClientDetails loadClientByClientId(String clientId) throws ClientRegistrationException {
        OauthClientDetails clientDetailsVO = oauthClientDetailsApi.getByClientId(clientId).getResult();
        return convert(clientDetailsVO);
    }

    @Override
    public void addClientDetails(ClientDetails clientDetails) throws ClientAlreadyExistsException {
        logger.info("addClientDetails");
    }

    @Override
    public void updateClientDetails(ClientDetails clientDetails) throws NoSuchClientException {
        logger.info("updateClientDetails");
    }

    @Override
    public void updateClientSecret(String clientId, String secret) throws NoSuchClientException {
        logger.info("updateClientSecret");
    }

    @Override
    public void removeClientDetails(String clientId) throws NoSuchClientException {
        logger.info("removeClientDetails");
    }

    @Override
    public List<ClientDetails> listClientDetails() {
        //调用租户端账号中心的凭证接口获取用户所有权限
        List<OauthClientDetails> list = oauthClientDetailsApi.listAll().getResult();
        List<ClientDetails> res = new ArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            list.forEach(details-> res.add(convert(details)));
        }
        return res;
    }

    private BaseClientDetails convert(OauthClientDetails client){
        BaseClientDetails clientDetails = new BaseClientDetails();
        clientDetails.setClientId(client.getClientId());
        clientDetails.setClientSecret(client.getClientSecret());
        clientDetails.setAccessTokenValiditySeconds(client.getAccessTokenValidity());
        clientDetails.setRefreshTokenValiditySeconds(client.getRefreshTokenValidity());
        if(!StringUtils.isEmpty(client.getScope())) {
            clientDetails.setScope(Arrays.asList(client.getScope().split(CommonConstant.STRING_SIGN_COMMA)));
        } else {
            clientDetails.setScope(Collections.emptySet());
        }
        if(!StringUtils.isEmpty(client.getAuthorizedGrantTypes())) {
            clientDetails.setAuthorizedGrantTypes(Arrays.asList(client.getAuthorizedGrantTypes().split(CommonConstant.STRING_SIGN_COMMA)));
        } else {
            clientDetails.setAuthorizedGrantTypes(Collections.emptyList());
        }

        if(!StringUtils.isEmpty(client.getWebServerRedirectUri())) {
            clientDetails.setRegisteredRedirectUri(new LinkedHashSet<>(Collections.singletonList(AuthConstant.DEFAULT_REDIRECT_URI)));
        } else {
            clientDetails.setRegisteredRedirectUri(Collections.emptySet());
        }

        if(!StringUtils.isEmpty(client.getAutoapprove())){
            String[] strAutoapproves = client.getAutoapprove().split(CommonConstant.STRING_SIGN_COMMA);
            HashSet<String> autoapprove = new HashSet<>(Arrays.asList(strAutoapproves));
            clientDetails.setAutoApproveScopes(autoapprove);
        }
        else {
            clientDetails.setAutoApproveScopes(Collections.emptySet());
        }
        return clientDetails;
    }

}
